import jinja2
import os
from .compiler import generate
import glob
import re

_lib_js = {}

def lib(minified=False):
    global _lib_js
    key = "minified" if minified else "normal"
    if key not in _lib_js:
        runtime = os.path.join(os.path.dirname(__file__),"lib/jinja2.runtime.min.js" if minified else "lib/jinja2.runtime.js")
        _lib_js[key] = open(runtime,'r').read()
    return _lib_js[key]

class JsJinja (object):
    js_environment = 'Jinja2'
    def __init__(self,environment=None):
        self.environment = environment or jinja2.Environment()

    def generate_node(self,node,name):
        return generate(node,self.environment,name,name,env=self.js_environment)

    def generate(self,filename):
        source, fn, _ = self.environment.loader.get_source(self.environment,filename)
        return self._generate(source,filename)

    def _generate(self,source,name):
        node = self.environment._parse(source,name,name)
        return self.generate_node(node,name)

    def generate_all(self):
        if not self.environment.loader:
            raise Exception("The Jinja2 environment doesn't have a template loader associated.\nYou must specify it for using the generate_all method.")
        templates = self.environment.list_templates()
        return ';'+';\n'.join(map(self.generate,templates))+';'

    def generate_source(self,source,name=None):
        return self._generate(source,name)

    lib = staticmethod(lib)

def generate_template():
    from optparse import OptionParser
    j = JsJinja()

    usage = "usage: %prog [options] files"
    parser = OptionParser(usage)
    parser.add_option("-o", "--output", dest="output",default=None,
                  help="write output to FILE", metavar="FILE")
    parser.add_option("-b", "--base", dest="base",default=None,
                  help="Set tempalte dir for dropping it in template name", metavar="FILE")
    parser.add_option("-l", "--lib", dest="lib",default=False,
                  help="Include Jinja2 runtime lib", action="store_true")
    (options, args) = parser.parse_args()

    if not args:
        raise Exception('You must specify input files')

    files = []
    for a in args:
        files += glob.glob(a)

    generated = [lib(True)] if options.lib else []
    for f in files:
        source = open(f).read()
        if options.base:
            f = re.sub('^'+options.base+'/?', '', f)
        gen = j.generate_source(source, f)
        generated.append(gen)

    generated = ';'+';\n'.join(generated)+';'
    output = options.output
    if output:
        with open(output,'w') as f:
            f.write(generated)
    else:
        print generated

# j = jsjinja()
# print j.generate('template.tmpl')



from jinja2 import nodes
from jinja2.ext import Extension
from jsjinja import JsJinja
# from generate import generate, generate_js, generate_node, generate_all_templates

class JsJinjaExtension(Extension):
    # a set of names that trigger the extension.
    tags = set(['jsjinja'])

    def __init__(self, environment):
        super(JsJinjaExtension, self).__init__(environment)

        # add the defaults to the environment
        environment.extend(
            jsjinja=JsJinja(self.environment)
            # generate_js=self.generate_js,
            # generate_all_templates=self.generate_all_templates,
        )
        
    # def generate_js(self, name=None, source=None):
    #     if not source and not name:
    #         raise Exception("You must specity the name or source (...).generate_js([name|source]=...)")
    #     if not source:
    #         source = generate_js(self.environment, name)
    #     return generate(self.environment, source, name)

    # def generate_all_templates(self):
    #     return generate_all_templates(self.environment)

    def parse(self, parser):
        parser.stream.next().lineno # lineno = 
        body = parser.parse_statements(['name:endjsjinja'], drop_needle=True)
        node = nodes.Template(body,lineno=1)
        code = self.environment.jsjinja.generate_node(node or body[0],None)
        return nodes.Output([nodes.Const(code)]).set_lineno(1)

    # def a(self,*args,**kwargs):
    #     return repr(args)+repr(kwargs)
    # #     # print type(body[0])
    # #     # lineno, body
    # #     node = nodes.Template(body,lineno=1)
    # #     generate_node(self.environment,node,'<none>')



#encoding: utf8
import os
import jinja2
import jsjinja

from nose import with_setup


def setup_func():
    global jinja_env, processors

def teardown_func():
    pass


from pyv8 import PyV8

TEMPLATE_FOLDER = 'templates/'
env = jinja2.Environment(loader = jinja2.FileSystemLoader(TEMPLATE_FOLDER))
env.add_extension('jsjinja.ext.JsJinjaExtension')

class Global(PyV8.JSClass): 
  def log(self,*args):
    print args



ctx = PyV8.JSContext(Global())
ctx.enter()

ctx.eval(jsjinja.lib())


# templates = env.list_templates()

def test_extension():
    js = env.jsjinja.generate_source(source='{{a}}')
    ex = ctx.eval('(new (%s))'%js).render({"a":"test"})
    assert ex == "test"

def test_extensiontag():
    template = '''{% jsjinja %}{% macro x(s) %}{{s}}{% endmacro %}{% endjsjinja %}'''
    t = env.from_string(template)
    js = str(t.render())
    print js
    ex = ctx.eval('(new (%s))'%js).module().x("test")
    assert ex == "test"

code = env.jsjinja.generate_all()
# raise Exception(code) 
ctx.eval(code)

context = {'context':True}


def compare_templates(f):
    jinja_template = env.get_template(f).render(context)
    # raise Exception(ctx.locals.Jinja2.templates[f])
    js_template = ctx.locals.Jinja2.getTemplate(f)
    js_template_rendered = unicode(js_template.render(context),'utf-8')
    print 'JS TEMPLATE:\n',js_template
    print 'Jinja:\n',jinja_template
    print 'Js:\n',js_template_rendered
    assert jinja_template == js_template_rendered

def test_case_generator():
    templates = env.list_templates()
    for f in templates:
        yield compare_templates, f

def main():
    """Runs the testsuite as command line application."""
    import nose
    try:
        nose.main(defaultTest="")
    except Exception, e:
        print 'Error: %s' % e

if __name__ == '__main__':
    main()
# for dirname, dirnames, filenames in os.walk(TEMPLATE_FOLDER):



# print gen2('''{% extends "a" %}
# {% block c %}
#   {% for i in x %}{{i}}:::{{loop.index}}
#   {% endfor %}
# {% endblock %}
# {% macro for1(datsa) -%}{{data}}{% endmacro %}
# {% block a %}{{ super() }}
#   {% include "x" %}
#   {{a|capitalize}}
#   a
#   {{for1(s)}}
# {% endblock %}''')

# print gen('''{% extends "a" %}''')
# t = '''{% set s=2 %}{% macro for1(data,dos,tres=3) -%}{{data}}{% set s=3233223 %}{{s}}{% endmacro %}{{for1(1)}}{{s}}'''
# print gen(t)
# template = env.from_string(t)
# print template.render()
# print gen('''{% from a import b %}''')
# print gen2('''{% extends 'a' %}''')


# print gen2('''{% set a = 2*2 or 7 %}{{a}}''')
# print gen2('''{% for a,b in x(c) if a==1 %}{{loop}}{% for s in sa %}{{loop}}{% endfor %}{% else %}s{% endfor %}''')

# filename = '%s%s'%(TEMPLATE_FOLDER,'include.tmpl')
# template_string = open(filename).read()
# code = gen2(template_string, filename)


# with open('runtime.js', 'r') as f:
#     ctx.eval(f.read())

# try:
#   # t = ctx.eval('new (%s)'%code).render(context)
#   t = ctx.eval(code)
#   # print code
#   js_template = ctx.locals.Jinja2.get(filename)
#   # print js_template.module().x(233)
#   s1 = env.from_string(template_string).render(context)
#   s2 = js_template.render(context)
# except Exception, e:
#   print code
#   raise e

# t = ctx.eval('new %s()'%code)
# print t.prototype.root({'vars':{}})
# print gen2('''{% extends 'a' %}{% set p=2 %}''')
# print gen2('''{% extends 'a' %}{% block a %}{{s}}{% endblock %}s{% block r %}{{super}}{% endblock %}''')
# print gen2('''{% set c=True %}{% if d or c %}{{a}}{% else %}{{d}}{% endif %}''')




# Based on http://www.dabeaz.com/generators/ and http://cburgmer.posterous.com/pip-requirementstxt-and-setuppy
import re
import os
import sys


PY_VER_STR = "py%d%d" % sys.version_info[:2]


def gen_find_applicable_requirements_filenames(base_filename):
    """
    Generator that yields the passed base requirements filename, along with its
    version specific requirements filename (if exists)
    """
    yield base_filename
    
    base, ext = os.path.splitext(base_filename)
    version_specific_filename = "%s.%s%s" % (base, PY_VER_STR, ext)
    if os.path.exists(version_specific_filename):
        yield version_specific_filename

def gen_open_files(filenames):
    """
    Generator that takes an iterable containing filenames as input and yields a
    sequence of file objects that have been suitably open
    """
    for name in filenames:
        yield open(name)

def gen_cat(sources):
    """
    Generator that concatenates multiple generators into a single sequence
    """
    for s in sources:
        for item in s:
            yield item

def gen_lines_from_requirements(base_filename):
    """
    Generator that finds all applicable requirements filenames and yields their
    contents, line at a time
    """
    filenames = gen_find_applicable_requirements_filenames(base_filename)
    files = gen_open_files(filenames)
    lines = gen_cat(files)
    return lines

def parse_requirements(base_filename):
    """
    Finds all applicable requirements filenames, parse them and return the
    requirements
    """
    requirements = []
    for line in gen_lines_from_requirements(base_filename):
        if re.match(r'(\s*#)|(\s*$)', line):
            continue
        if re.match(r'\s*-e\s+', line):
            requirements.append(re.sub(r'\s*-e\s+.*#egg=(.*)$', r'\1', line))
        elif re.match(r'\s*-f\s+', line):
            pass
        else:
            requirements.append(line)
    
    return requirements

def parse_dependency_links(base_filename):
    """
    Finds all applicable requirements filenames, parse them and return the
    dependency links
    """
    dependency_links = []
    for line in gen_lines_from_requirements(base_filename):
        if re.match(r'\s*-[ef]\s+', line):
            dependency_links.append(re.sub(r'\s*-[ef]\s+', '', line))
    
    return dependency_links



# from distribute_setup import use_setuptools
# use_setuptools() 

from setuptools import setup, find_packages
from requirements_utils import parse_dependency_links, parse_requirements

print find_packages()
setup(
    name='jsjinja',
    version='0.3.1',
    url='https://github.com/syrusakbary/jsjinja',
    download_url = 'git@github.com:syrusakbary/jsjinja.git',
    author='Syrus Akbary',
    author_email='me@syrusakbary.com',
    description='Jinja2 to Javascript compiler',
    long_description=open('README.rst').read(),
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Scientific/Engineering',
    ],
    platforms='any',
    packages=find_packages(),
    keywords='jinja2 javascript converter coffeescript',
    include_package_data=True,
    entry_points={
        'console_scripts' : ['jsjinja = jsjinja:generate_template',]
    },
    install_requires = parse_requirements('requirements.txt'),
    dependency_links = parse_dependency_links('requirements.txt'),
    # setup_requires = ['nose>=1.0'],
    tests_require = parse_requirements('requirements-test.txt'),
    # test_dirs='jsjinja/testsuite',
    test_suite = "nose.collector"
)
